<?php
namespace metagen_php;
use Exception;

function flt_str2num($val, $name, $struct, $args)
{
  if(is_string($val))
  {
    if(strlen($val) === 0)
      throw new Exception("Bad value, string empty, crc28 can't be generated");

    //special case
    if(strpos($val, '@') !== 0)
      throw new Exception("@ expected");

    $name = substr($val, 1) . '.conf.js';

    $path = \taskman\config_real_path($name);

    if(\taskman\config_get_header($path, $proto_id, $alias))
    {
      if($alias !== $val)
        throw new Exception("Config file header alias doesn't match: $alias vs $val");
      $val = $proto_id;
    }
    else
     throw new Exception("Config file header is not valid: $path");
  }

  if(!is_numeric($val))
    throw new Exception("Bad value, not a number(" . serialize($val) . ")");

  return 1*$val;
}

function flt_range($val, $name, $struct, $args)
{
  if(count($args) != 2)
    throw new Exception("Wrong usage of \"flt_range\". Use \"@flt_range:[0,1]\" where 0 - min, 1 - max");
    
  if(!is_numeric($args[0]) || !is_numeric($args[1]))
      throw new Exception("Wrong usage of \"flt_range\". Use \"@flt_range:[0,1]\" where 0 - min, 1 - max");

  if(is_numeric($val))
  {
    if($val < $args[0] || $val > $args[1])
      throw new Exception("Value ".$val.", not in range(" . $args[0] . "," . $args[1] . ")");
  }

  return $val;
}

function flt_time($val, $name, $struct, $args)
{
  $str = "";
  if(is_string($val))
  {
    if(strlen($val) === 0)
      throw new Exception("Bad value, string empty, crc28 can't be generated");

    $reg = "/^([0-9]{1,2}):([0-9]{1,2}):([0-9]{1,2})$/";
    
    if(!preg_match($reg, $val, $matches))
      throw new Exception("Bad value, pattern 00:00:00");

    $h = intval($matches[1]);
    $m = intval($matches[2]);
    $s = intval($matches[3]);
    
    return $h * 3600 + $m * 60 + $s;
  }

  return 0;
}

function flt_class($val, $name, $struct, $args)
{
  global $GAME_ROOT;
  
  if(is_string($val))
  {
    $conf_path = $val;
    
    if(strpos($conf_path, '@') === 0)
      $conf_path = substr($conf_path, 1) . '.conf.js';

    $conf_path = \taskman\config_real_path($conf_path);
    
    $cce = \taskman\config_fetch_by_path($conf_path);

    $conf_class = $cce->class;

    $target_class = $args;
    if(is_array($target_class))
      $target_class = $target_class[0];

    if(!class_exists($target_class))
      throw new Exception($val.". Target class \"".$target_class."\" is not valid");
    
    if($conf_class != $target_class && !is_subclass_of($conf_class, $target_class))
      throw new Exception($val.". Config class is \"".$conf_class."\". Must be \"".$target_class."\" or it's child");
  }
  
  return $val;
}
